// SPDX-License-Identifier: AGPL-3.0-or-later
// SPDX-FileCopyrightText: 2020-present grommunio GmbH

import React, { PureComponent } from "react";
import PropTypes from "prop-types";
import { withStyles } from "@mui/styles";
import { withTranslation } from "react-i18next";
import { Checkbox, IconButton, Paper, Table, TableBody, TableCell, TableHead, TableRow, Tooltip, Typography } from "@mui/material";
import { connect } from "react-redux";
import { deleteMailQData, fetchMailQData, flushMailQData, requeueMailQData } from "../actions/mailq";
import TableViewContainer from "../components/TableViewContainer";
import { parseUnixtime } from "../utils";
import { Delete, Replay, PlayForWork } from "@mui/icons-material";

const styles = (theme) => ({
  logViewer: {
    flex: 1,
  },
  log: {
    fontSize: 16,
    cursor: 'pointer',
    '&:hover': {
      backgroundColor: '#bbb',
    },
  },
  paper: {
    flex: 1,
    padding: theme.spacing(2, 2, 2, 2),
  },
  divider: {
    margin: theme.spacing(2, 0, 2, 0),
  },
  header: {
    marginBottom: 8,
  },
  flexRow: {
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'flex-end',
  },
});

class MailQ extends PureComponent {

  state = {
    selected: [],
    postfixMailq: '',
    gromoxMailq: '',
    postqueue: [],
    snackbar: '',
    loading: true,
  };

  fetchInterval = null;

  async componentDidMount() {
    this.fetchData();
    this.fetchInterval = setInterval(() => {
      this.fetchData();
    }, 10000);
  }

  handleNavigation = (path) => (event) => {
    const { history } = this.props;
    event.preventDefault();
    history.push(`/${path}`);
  };

  fetchData = async () => {
    const data = await this.props.fetch()
      .catch(snackbar => this.setState({ snackbar }));
    if(data) {
      this.setState({ ...data, loading: false });
    }
  }

  componentWillUnmount() {
    clearInterval(this.fetchInterval);
  }

  handleFlush = () => {
    const { flush } = this.props;
    flush(this.state.selected)
      .then(() => this.setState({ snackbar: 'Success!', selected: [] }))
      .catch(snackbar => this.setState({ snackbar }));
  }

  handleDelete = () => {
    const { selected, postqueue } = this.state;
    const { deleteQ } = this.props;
    deleteQ(selected.length === postqueue.length  ? 'ALL' : selected.join(','))
      .then(() => this.setState({ snackbar: 'Success!', selected: [] }))
      .catch(snackbar => this.setState({ snackbar }));
  }

  handleRequeue = () => {
    const { selected, postqueue } = this.state;
    const { requeue } = this.props;
    requeue(selected.length === postqueue.length  ? 'ALL' : selected.join(','))
      .then(() => this.setState({ snackbar: 'Success!', selected: [] }))
      .catch(snackbar => this.setState({ snackbar }));
  }

  handleCheckAll = ({ target: t }) => this.setState({
    selected: t.checked ? this.state.postqueue.map(entry => entry.queue_id) : [],
  });

  handleCheckbox = qID => (e) => {
    const copy = [...this.state.selected];
    if(e.target.checked) {
      copy.push(qID);
    } else {
      copy.splice(copy.findIndex(el => el === qID), 1);
    }
    this.setState({ selected: copy });
  };

  render() {
    const { classes, t } = this.props;
    const { selected, snackbar, gromoxMailq, postqueue, loading } = this.state;
    const actionsDisabled = selected.length === 0;
    return (
      <TableViewContainer
        headline={t("Mail queue")}
        subtitle={t("mailq_sub")}
        href="https://docs.grommunio.com/admin/administration.html#mail-queue"
        snackbar={snackbar}
        onSnackbarClose={() => this.setState({ snackbar: '' })}
        loading={loading}
      >  
        <div className={classes.logViewer}>
          <Paper elevation={1} className={classes.paper}>
            <Typography variant="h6" className={classes.header}>Postfix {t("mail queue")}</Typography>
            <Table size="small">
              <TableHead>
                <TableRow>
                  <TableCell padding="checkbox">
                    <Checkbox
                      checked={postqueue.length > 0 && postqueue.length === selected.length}
                      onClick={this.handleCheckAll}
                    />
                  </TableCell>
                  <TableCell>{t("Queue ID")}</TableCell>
                  <TableCell>{t("Queue name")}</TableCell>
                  <TableCell>{t("Arrival time")}</TableCell>
                  <TableCell>{t("Sender")}</TableCell>
                  <TableCell>{t("Recipients")}</TableCell>
                  <TableCell className={classes.flexRow}>
                    <Tooltip title={t("Flush mail queue")}>
                      <span>
                        <IconButton disabled={actionsDisabled} onClick={this.handleFlush}>
                          <PlayForWork color={actionsDisabled ? "secondary" : "primary"} />
                        </IconButton>
                      </span>
                    </Tooltip>
                    <Tooltip title={t("Requeue")}>
                      <span>
                        <IconButton disabled={actionsDisabled} onClick={this.handleRequeue}>
                          <Replay color={actionsDisabled ? "secondary" : "warning"} style={{ transform: 'rotate(180deg)' }} />
                        </IconButton>
                      </span>
                    </Tooltip>
                    <Tooltip title={t("Delete")}>
                      <span>
                        <IconButton disabled={actionsDisabled} onClick={this.handleDelete}>
                          <Delete color={actionsDisabled ? "secondary" : "error"} />
                        </IconButton>
                      </span>
                    </Tooltip>
                  </TableCell>
                </TableRow>
              </TableHead>
              <TableBody>
                {postqueue.map(entry => {
                  const id = entry.queue_id;
                  const rowSelected = selected.includes(id)
                  return (
                    <TableRow key={entry.queue_id}>
                      <TableCell padding="checkbox">
                        <Checkbox
                          onClick={this.handleCheckbox(id)}
                          checked={rowSelected}
                          value={rowSelected}
                        />
                      </TableCell>
                      <TableCell>{id}</TableCell>
                      <TableCell>{entry.queue_name}</TableCell>
                      <TableCell>{parseUnixtime(entry.arrival_time)}</TableCell>
                      <TableCell>{entry.sender}</TableCell>
                      <TableCell>
                        {(entry.recipients || []).map(r => <p key={r.address}>{r.address + ' (' + r.delay_reason + ')'}</p>)}
                      </TableCell>
                      <TableCell></TableCell>
                    </TableRow>
                  );
                })}
              </TableBody>
            </Table>
          </Paper>
          <Paper elevation={1} className={classes.paper}>
            <Typography variant="h6" className={classes.header}>Gromox {t("mail queue")}</Typography>
            <pre
              className={classes.log}
            >
              {gromoxMailq}
            </pre>
          </Paper>
        </div>
      </TableViewContainer>
    );
  }
}

MailQ.propTypes = {
  classes: PropTypes.object.isRequired,
  t: PropTypes.func.isRequired,
  history: PropTypes.object.isRequired,
  fetch: PropTypes.func.isRequired,
  flush: PropTypes.func.isRequired,
  deleteQ: PropTypes.func.isRequired,
  requeue: PropTypes.func.isRequired,
};

const mapDispatchToProps = (dispatch) => {
  return {
    fetch: async () => await dispatch(fetchMailQData())
      .catch((error) => Promise.reject(error)),
    flush: async qIDs => await dispatch(flushMailQData(qIDs))
      .catch((error) => Promise.reject(error)),
    deleteQ: async qID => await dispatch(deleteMailQData({ queue: qID }))
      .catch((error) => Promise.reject(error)),
    requeue: async qID => await dispatch(requeueMailQData({ queue: qID }))
      .catch((error) => Promise.reject(error)),
  };
};

export default connect(
  null,
  mapDispatchToProps
)(withTranslation()(withStyles(styles)(MailQ)));
